@@ -1,9 +1,9 @@
-use crate::check_var::{check_vars, check_vars_in_rewriter};
+use crate::check_var::{check_rule_with_hint, CheckHint};
 use crate::fixer::{Fixer, FixerError, SerializableFixer};
 use crate::rule::referent_rule::RuleRegistration;
 use crate::rule::Rule;
 use crate::rule::{RuleSerializeError, SerializableRule};
-use crate::transform::{apply_env_transform, Transformation};
+use crate::transform::{Transform, TransformError, Transformation};
 use crate::DeserializeEnv;
 
 use ast_grep_core::language::Language;
@@ -21,24 +21,24 @@ use std::collections::{HashMap, HashSet};
 use std::ops::Deref;
 
 #[derive(Debug, Error)]
-pub enum RuleConfigError {
+pub enum RuleCoreError {
   #[error("Fail to parse yaml as RuleConfig")]
   Yaml(#[from] YamlError),
-  #[error("Rule is not configured correctly.")]
-  Rule(#[from] RuleSerializeError),
-  #[error("Utility rule is not configured correctly.")]
+  #[error("`utils` is not configured correctly.")]
   Utils(#[source] RuleSerializeError),
-  #[error("fix pattern is invalid.")]
-  Fixer(#[from] FixerError),
-  #[error("constraints is not configured correctly.")]
+  #[error("`rule` is not configured correctly.")]
+  Rule(#[from] RuleSerializeError),
+  #[error("`constraints` is not configured correctly.")]
   Constraints(#[source] RuleSerializeError),
-  #[error("Undefined meta var {0} used in {1}.")]
+  #[error("`transform` is not configured correctly.")]
+  Transform(#[from] TransformError),
+  #[error("`fix` pattern is invalid.")]
+  Fixer(#[from] FixerError),
+  #[error("Undefined meta var `{0}` used in `{1}`.")]
   UndefinedMetaVar(String, &'static str),
-  #[error("Undefined rewriter {0} used in transform.")]
-  UndefinedRewriter(String),
 }
 
-type RResult<T> = std::result::Result<T, RuleConfigError>;
+type RResult<T> = std::result::Result<T, RuleCoreError>;
 
 /// Used for global rules, rewriters, and pyo3/napi
 #[derive(Serialize, Deserialize, Clone, JsonSchema)]
@@ -67,7 +67,7 @@ impl SerializableRuleCore {
     if let Some(utils) = &self.utils {
       let env = env
         .register_local_utils(utils)
-        .map_err(RuleConfigError::Utils)?;
+        .map_err(RuleCoreError::Utils)?;
       Ok(env)
     } else {
       Ok(env)
@@ -83,7 +83,10 @@ impl SerializableRuleCore {
       return Ok(constraints);
     };
     for (key, ser) in serde_cons {
-      constraints.insert(key.to_string(), env.deserialize_rule(ser.clone())?);
+      let constraint = env
+        .deserialize_rule(ser.clone())
+        .map_err(RuleCoreError::Constraints)?;
+      constraints.insert(key.to_string(), constraint);
     }
     Ok(constraints)
   }
@@ -100,7 +103,11 @@ impl SerializableRuleCore {
   fn get_matcher_from_env<L: Language>(&self, env: &DeserializeEnv<L>) -> RResult<RuleCore<L>> {
     let rule = env.deserialize_rule(self.rule.clone())?;
     let constraints = self.get_constraints(env)?;
-    let transform = self.transform.clone();
+    let transform = self
+      .transform
+      .as_ref()
+      .map(|t| Transform::deserialize(t, env))
+      .transpose()?;
     let fixer = self.get_fixer(env)?;
     Ok(
       RuleCore::new(rule)
@@ -112,26 +119,22 @@ impl SerializableRuleCore {
   }
 
   pub fn get_matcher<L: Language>(&self, env: DeserializeEnv<L>) -> RResult<RuleCore<L>> {
-    let env = self.get_deserialize_env(env)?;
-    let ret = self.get_matcher_from_env(&env)?;
-    check_vars(&ret.rule, &ret.constraints, &ret.transform, &ret.fixer)?;
-    Ok(ret)
+    self.get_matcher_with_hint(env, CheckHint::Normal)
   }
 
-  /// this is needed for checking metavar defined in containing vars
-  pub(crate) fn get_rewriter<L: Language>(
+  pub(crate) fn get_matcher_with_hint<L: Language>(
     &self,
     env: DeserializeEnv<L>,
-    upper_vars: &HashSet<&str>,
+    hint: CheckHint,
   ) -> RResult<RuleCore<L>> {
     let env = self.get_deserialize_env(env)?;
     let ret = self.get_matcher_from_env(&env)?;
-    check_vars_in_rewriter(
+    check_rule_with_hint(
       &ret.rule,
       &ret.constraints,
-      &ret.transform,
+      &self.transform,
       &ret.fixer,
-      upper_vars,
+      hint,
     )?;
     Ok(ret)
   }
@@ -141,7 +144,7 @@ pub struct RuleCore<L: Language> {
   rule: Rule<L>,
   constraints: HashMap<String, Rule<L>>,
   kinds: Option<BitSet>,
-  pub(crate) transform: Option<HashMap<String, Transformation>>,
+  pub(crate) transform: Option<Transform>,
   pub fixer: Option<Fixer<L>>,
   // this is required to hold util rule reference
   utils: RuleRegistration<L>,
@@ -172,7 +175,7 @@ impl<L: Language> RuleCore<L> {
   }
 
   #[inline]
-  pub fn with_transform(self, transform: Option<HashMap<String, Transformation>>) -> Self {
+  pub fn with_transform(self, transform: Option<Transform>) -> Self {
     Self { transform, ..self }
   }
 
@@ -222,14 +225,14 @@ impl<L: Language> RuleCore<L> {
       return None;
     }
     if let Some(trans) = &self.transform {
-      let lang = ret.lang();
       let rewriters = self.utils.get_rewriters();
+      let rewriters = rewriters.read();
       let env = env.to_mut();
       if let Some(enclosing) = enclosing_env {
-        apply_env_transform(trans, lang, env, rewriters, enclosing);
+        trans.apply_transform(env, &rewriters, enclosing);
       } else {
         let enclosing = env.clone();
-        apply_env_transform(trans, lang, env, rewriters, &enclosing);
+        trans.apply_transform(env, &rewriters, &enclosing);
       };
     }
     Some(ret)
@@ -273,11 +276,62 @@ impl<L: Language> Matcher<L> for RuleCore<L> {
 #[cfg(test)]
 mod test {
   use super::*;
-  use crate::rule::referent_rule::ReferentRule;
+  use crate::rule::referent_rule::{ReferentRule, ReferentRuleError};
   use crate::test::TypeScript;
   use crate::{from_str, GlobalRules};
   use ast_grep_core::matcher::{Pattern, RegexMatcher};
 
+  fn get_matcher(src: &str) -> RResult<RuleCore<TypeScript>> {
+    let env = DeserializeEnv::new(TypeScript::Tsx);
+    let rule: SerializableRuleCore = from_str(src).expect("should word");
+    rule.get_matcher(env)
+  }
+
+  #[test]
+  fn test_rule_error() {
+    let ret = get_matcher(r"rule: {kind: bbb}");
+    assert!(matches!(ret, Err(RuleCoreError::Rule(_))));
+  }
+
+  #[test]
+  fn test_utils_error() {
+    let ret = get_matcher(
+      r"
+rule: { kind: number }
+utils: { testa: {kind: bbb} }
+  ",
+    );
+    assert!(matches!(ret, Err(RuleCoreError::Utils(_))));
+  }
+
+  #[test]
+  fn test_undefined_utils_error() {
+    let ret = get_matcher(r"rule: { kind: number, matches: undefined-util }");
+    match ret {
+      Err(RuleCoreError::Rule(RuleSerializeError::MatchesReference(
+        ReferentRuleError::UndefinedUtil(name),
+      ))) => {
+        assert_eq!(name, "undefined-util");
+      }
+      _ => panic!("wrong error"),
+    }
+  }
+
+  #[test]
+  fn test_cyclic_transform_error() {
+    let ret = get_matcher(
+      r"
+rule: { kind: number }
+transform:
+  A: {substring: {source: $B}}
+  B: {substring: {source: $A}}",
+    );
+    assert!(matches!(
+      ret,
+      Err(RuleCoreError::Transform(TransformError::Cyclic(_)))
+    ));
+  }
+
   #[test]
   fn test_rule_reg_with_utils() {
     let env = DeserializeEnv::new(TypeScript::Tsx);
