@@ -514,4 +514,15 @@ mod test {
     let vars = defined_vars("var $A = $A");
     assert_eq!(vars, ["A"]);
   }
+
+  #[test]
+  fn test_contextual_pattern_vars() {
+    let pattern = Pattern::contextual("<div ref={$A}/>", "jsx_attribute", Tsx).expect("correct");
+    assert_eq!(pattern.defined_vars(), ["A"].into_iter().collect());
+  }
+
+  #[test]
+  fn test_gh_1087() {
+    test_match("($P) => $F($P)", "(x) => bar(x)");
+  }
 }
