@@ -379,10 +379,6 @@ impl<'r, D: Doc> Node<'r, D> {
   // NOTE: Need go to parent first, then move to current node by byte offset.
   // This is because tree_sitter cursor is scoped to the starting node.
   // See https://github.com/tree-sitter/tree-sitter/issues/567
-  // TODO: remove this target_arch dependent code branch
-  // TODO: we need to add goto_first_child_for_byte in web-tree-sitter
-  // https://github.com/tree-sitter/tree-sitter/discussions/2288
-  #[cfg(not(target_arch = "wasm32"))]
   pub fn next_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
     // if root is none, use self as fallback to return a type-stable Iterator
     let node = self.parent().unwrap_or_else(|| self.clone());
@@ -397,20 +393,6 @@ impl<'r, D: Doc> Node<'r, D> {
     })
   }
 
-  // TODO: remove this target_arch dependent code branch
-  // TODO: we need to add goto_first_child_for_byte in web-tree-sitter
-  // https://github.com/tree-sitter/tree-sitter/discussions/2288
-  #[cfg(target_arch = "wasm32")]
-  pub fn next_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    let mut node = self.clone();
-    std::iter::from_fn(move || {
-      node.next().map(|n| {
-        node = n.clone();
-        n
-      })
-    })
-  }
-
   #[must_use]
   pub fn prev(&self) -> Option<Node<'r, D>> {
     let inner = self.inner.prev_sibling()?;
@@ -420,15 +402,17 @@ impl<'r, D: Doc> Node<'r, D> {
     })
   }
 
-  // TODO: use cursor to optimize clone.
-  // investigate why tree_sitter cursor cannot goto next_sibling
   pub fn prev_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    let mut node = self.clone();
+    // if root is none, use self as fallback to return a type-stable Iterator
+    let node = self.parent().unwrap_or_else(|| self.clone());
+    let mut cursor = node.inner.walk();
+    cursor.goto_first_child_for_byte(self.inner.start_byte());
     std::iter::from_fn(move || {
-      node.prev().map(|n| {
-        node = n.clone();
-        n
-      })
+      if cursor.goto_previous_sibling() {
+        Some(self.root.adopt(cursor.node()))
+      } else {
+        None
+      }
     })
   }
 
